@@ -4,6 +4,7 @@
  * Parse LaTeX code.
  *
  * Copyright (c) Chris Putnam 2020
+ * Copyright (c) Georgi N. Boshnakov 2020
  *
  * Source code released under the GPL version 2
  *
@@ -71,7 +72,9 @@ build_latex_graph_r( str *in, unsigned long *offset, int *mathmode, int depth, l
 	int status = BIBL_OK;
 	latex_edge *newedge;
 	char *p;
-
+// testing
+// REprintf( "build_latex_graph_r: '%s'\n", str_cstr( in ) );
+ 
 	newnode = latex_node_new();
 	if ( !newnode ) return BIBL_ERR_MEMERR;
 
@@ -168,10 +171,15 @@ build_latex_graph( str *in, latex_node **start )
 {
 	unsigned long offset = 0;
 	int mathmode = 0;
-	latex_node *n;
-
-	n = latex_node_new();
-	if ( !n ) return BIBL_ERR_MEMERR;
+	// latex_node *n;
+	// 
+	// // Georgi: I don't understand this - n is not used and not freed
+	// //         Is this a leftover or just a check that there is space for a node?
+	// //     Commenting out. TODO: this is dangerous if I am missing something!!!
+	// n = latex_node_new();
+	// if ( !n ) return BIBL_ERR_MEMERR;
+	//
+	// REprintf( "build_latex_graph: '%s'\n", str_cstr( in ) );
 	
 	return build_latex_graph_r( in, &offset, &mathmode, 0, start );
 }
@@ -321,24 +329,47 @@ write_latex_graph( latex_node *n )
 }
 #endif
 
+// Georgi: new functions - recursively free memory for nodes;
+static void latex_edge_delete_recursively( latex_edge *e ); // declaration
+
+static void
+latex_node_delete_recursively( latex_node *n )
+{
+  if( n->next_edge ) latex_edge_delete_recursively(n->next_edge);
+  if( n->down_node ) latex_node_delete_recursively( n->down_node );
+  free( n );
+}
+static void
+latex_edge_delete_recursively( latex_edge *e )
+{
+  if( e->next_node ) latex_node_delete_recursively( e->next_node );
+  latex_edge_delete(e);
+}
+
+
 int
 latex_parse( str *in, str *out )
 {
 	latex_node *n;
 	int status;
+	// REprintf( "latex_parse: '%s'\n", str_cstr( in ) );
 
 	str_empty( out );
 	if ( str_is_empty( in ) ) return BIBL_OK;
 
+	// Georgi: the code below was not clearing n, leading to memory leaks
 	status = build_latex_graph( in, &n );
-	if ( status!=BIBL_OK ) return status;
+	if ( status!=BIBL_OK ) goto out;
 
 	status = string_from_latex_graph( n, out );
-	if ( status!=BIBL_OK ) return status;
+	if ( status!=BIBL_OK ) goto out;
 
 	str_trimendingws( out );
 
-	return BIBL_OK;
+ out:
+	// Georgi: TODO: this assumes that the strings have been copied to out
+	latex_node_delete_recursively( n );
+	return status;
 }
 
 int
@@ -382,3 +413,4 @@ out:
 	str_free( &tok );
 	return status;
 }
+
